@@ -2,22 +2,9 @@
     kmime_header_parsing.cpp
 
     KMime, the KDE Internet mail/usenet news message library.
-    Copyright (c) 2001-2002 Marc Mutz <mutz@kde.org>
+    SPDX-FileCopyrightText: 2001-2002 Marc Mutz <mutz@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kmime_header_parsing.h"
@@ -49,6 +36,16 @@ using namespace KMime::Types;
 namespace KMime
 {
 
+    namespace Types
+    {
+        // Optimization to avoid allocating QStrings when the value isn't encoded
+        struct KMIME_EXPORT QStringOrQPair {
+            QStringOrQPair() : qstring(), qpair(nullptr, 0) {}
+            QString qstring;
+            QPair<const char *, int> qpair;
+        };
+    } // namespace Types
+
 namespace HeaderParsing
 {
 
@@ -247,12 +244,12 @@ static inline void eatWhiteSpace(const char *&scursor, const char *const send)
 }
 
 bool parseAtom(const char*&scursor, const char *const send,
-               QString &result, bool allow8Bit)
+               QByteArray &result, bool allow8Bit)
 {
     QPair<const char *, int> maybeResult;
 
     if (parseAtom(scursor, send, maybeResult, allow8Bit)) {
-        result = QString::fromLatin1(maybeResult.first, maybeResult.second);
+        result = QByteArray(maybeResult.first, maybeResult.second);
         return true;
     }
 
@@ -287,15 +284,13 @@ bool parseAtom(const char*&scursor, const char *const send,
     return success;
 }
 
-// FIXME: Remove this and the other parseToken() method. add a new one where "result" is a
-//        QByteArray.
 bool parseToken(const char*&scursor, const char *const send,
-                QString &result, ParseTokenFlags flags)
+                QByteArray &result, ParseTokenFlags flags)
 {
     QPair<const char *, int> maybeResult;
 
     if (parseToken(scursor, send, maybeResult, flags)) {
-        result = QString::fromLatin1(maybeResult.first, maybeResult.second);
+        result = QByteArray(maybeResult.first, maybeResult.second);
         return true;
     }
 
@@ -343,8 +338,6 @@ bool parseToken(const char*&scursor, const char *const send,
 //
 // - doesn't handle quoted CRLF
 
-// FIXME: Why is result a QString? This should be a QByteArray, since at this level, we don't
-//        know about encodings yet!
 bool parseGenericQuotedString(const char *&scursor, const char *const send,
                               QString &result, bool isCRLF,
                               const char openChar, const char closeChar)
@@ -559,6 +552,7 @@ bool parsePhrase(const char *&scursor, const char *const send,
 
     QString tmp;
     QByteArray lang, charset;
+    QPair<const char *, int> tmpAtom;
     const char *successfullyParsed = nullptr;
     // only used by the encoded-word branch
     const char *oldscursor;
@@ -665,9 +659,8 @@ bool parsePhrase(const char *&scursor, const char *const send,
             // fall though...
 
         default: //atom
-            tmp.clear();
             scursor--;
-            if (parseAtom(scursor, send, tmp, true /* allow 8bit */)) {
+            if (parseAtom(scursor, send, tmpAtom, true /* allow 8bit */)) {
                 successfullyParsed = scursor;
                 switch (found) {
                 case None:
@@ -684,7 +677,7 @@ bool parsePhrase(const char *&scursor, const char *const send,
                     assert(0);
                 }
                 lastWasEncodedWord = false;
-                result += tmp;
+                result += QLatin1String(tmpAtom.first, tmpAtom.second);
             } else {
                 if (found == None) {
                     return false;
@@ -700,16 +693,15 @@ bool parsePhrase(const char *&scursor, const char *const send,
     return found != None;
 }
 
-// FIXME: This should probably by QByteArray &result instead?
 bool parseDotAtom(const char *&scursor, const char *const send,
-                  QString &result, bool isCRLF)
+                  QByteArray &result, bool isCRLF)
 {
     eatCFWS(scursor, send, isCRLF);
 
     // always points to just after the last atom parsed:
     const char *successfullyParsed;
 
-    QString tmp;
+    QByteArray tmp;
     if (!parseAtom(scursor, send, tmp, false /* no 8bit */)) {
         return false;
     }
@@ -733,13 +725,13 @@ bool parseDotAtom(const char *&scursor, const char *const send,
         }
 
         // try to parse the next atom:
-        QString maybeAtom;
+        QByteArray maybeAtom;
         if (!parseAtom(scursor, send, maybeAtom, false /*no 8bit*/)) {
             scursor = successfullyParsed;
             return true;
         }
 
-        result += QLatin1Char('.');
+        result += '.';
         result += maybeAtom;
         successfullyParsed = scursor;
     }
@@ -822,14 +814,14 @@ bool parseDomain(const char *&scursor, const char *const send,
         }
     } else {
         // dot-atom:
-        QString maybeDotAtom;
+        QByteArray maybeDotAtom;
         if (parseDotAtom(scursor, send, maybeDotAtom, isCRLF)) {
-            result = maybeDotAtom;
             // Domain may end with '.', if so preserve it'
             if (scursor != send && *scursor == '.') {
-                result += QLatin1Char('.');
+                maybeDotAtom += '.';
                 scursor++;
             }
+            result = QString::fromLatin1(maybeDotAtom);
             return true;
         }
     }
@@ -907,6 +899,7 @@ bool parseAddrSpec(const char *&scursor, const char *const send,
 
     QString maybeLocalPart;
     QString tmp;
+    QPair<const char *, int> tmpAtom;
 
     while (scursor != send) {
         // first, eat any whitespace
@@ -933,9 +926,8 @@ bool parseAddrSpec(const char *&scursor, const char *const send,
 
         default: // atom
             scursor--; // re-set scursor to point to ch again
-            tmp.clear();
-            if (parseAtom(scursor, send, tmp, false /* no 8bit */)) {
-                maybeLocalPart += tmp;
+            if (parseAtom(scursor, send, tmpAtom, false /* no 8bit */)) {
+                maybeLocalPart += QLatin1String(tmpAtom.first, tmpAtom.second);
             } else {
                 return false; // parseAtom can only fail if the first char is non-atext.
             }
@@ -1229,11 +1221,8 @@ bool parseAddressList(const char *&scursor, const char *const send,
     return true;
 }
 
-// FIXME: Get rid of the very ugly "QStringOrQPair" thing. At this level, we are supposed to work
-//        on byte arrays, not strings! The result parameter should be a simple
-//        QPair<QByteArray,QByteArray>, which is the attribute name and the value.
-bool parseParameter(const char *&scursor, const char *const send,
-                    QPair<QString, QStringOrQPair> &result, bool isCRLF)
+static bool parseParameter(const char *&scursor, const char *const send,
+                           QPair<QString, QStringOrQPair> &result, bool isCRLF)
 {
     // parameter = regular-parameter / extended-parameter
     // regular-parameter = regular-parameter-name "=" value
@@ -1241,7 +1230,7 @@ bool parseParameter(const char *&scursor, const char *const send,
     // value = token / quoted-string
     //
     // note that rfc2231 handling is out of the scope of this function.
-    // Therefore we return the attribute as QString and the value as
+    // Therefore we return the attribute as QByteArray and the value as
     // (start,length) tupel if we see that the value is encoded
     // (trailing asterisk), for parseParameterList to decode...
 
@@ -1253,11 +1242,12 @@ bool parseParameter(const char *&scursor, const char *const send,
     //
     // parse the parameter name:
     //
-    // FIXME: maybeAttribute should be a QByteArray
-    QString maybeAttribute;
-    if (!parseToken(scursor, send, maybeAttribute, ParseTokenNoFlag)) {
+    QByteArray tmpAttr;
+    if (!parseToken(scursor, send, tmpAttr, ParseTokenNoFlag)) {
         return false;
     }
+    // FIXME: we could use QMap<QByteArray, ...> in the API for parameters
+    QString maybeAttribute = QString::fromLatin1(tmpAttr);
 
     eatCFWS(scursor, send, isCRLF);
     // premature end: not OK (haven't seen '=' yet).
@@ -1314,11 +1304,9 @@ bool parseParameter(const char *&scursor, const char *const send,
     return true;
 }
 
-// FIXME: Get rid of QStringOrQPair: Use a simply QMap<QByteArray, QByteArray> for "result"
-//        instead!
-bool parseRawParameterList(const char *&scursor, const char *const send,
-                           QMap<QString, QStringOrQPair> &result,
-                           bool isCRLF)
+static bool parseRawParameterList(const char *&scursor, const char *const send,
+                                  QMap<QString, QStringOrQPair> &result,
+                                  bool isCRLF)
 {
     // we use parseParameter() consecutively to obtain a map of raw
     // attributes to raw values. "Raw" here means that we don't do
