@@ -2,24 +2,11 @@
     kmime_headers.cpp
 
     KMime, the KDE Internet mail/usenet news message library.
-    Copyright (c) 2001-2002 the KMime authors.
+    SPDX-FileCopyrightText: 2001-2002 the KMime authors.
     See file AUTHORS for details
-    Copyright (c) 2006 Volker Krause <vkrause@kde.org>
+    SPDX-FileCopyrightText: 2006 Volker Krause <vkrause@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 /**
   @file
@@ -44,7 +31,6 @@
 #include "kmime_util_p.h"
 #include "kmime_codecs.h"
 #include "kmime_content.h"
-#include "kmime_header_parsing.h"
 #include "kmime_headerfactory_p.h"
 #include "kmime_debug.h"
 #include "kmime_warning.h"
@@ -53,8 +39,6 @@
 #include <KCodecs>
 
 #include <QTextCodec>
-#include <QString>
-#include <QStringList>
 
 #include <assert.h>
 #include <ctype.h>
@@ -800,13 +784,13 @@ QByteArray DotAtom::as7BitString(bool withHeaderType) const
         rv += typeIntro();
     }
 
-    rv += d_func()->dotAtom.toLatin1(); // FIXME: encoding?
+    rv += d_func()->dotAtom;
     return rv;
 }
 
 QString DotAtom::asUnicodeString() const
 {
-    return d_func()->dotAtom;
+    return QString::fromLatin1(d_func()->dotAtom);
 }
 
 void DotAtom::clear()
@@ -824,7 +808,7 @@ bool DotAtom::parse(const char *&scursor, const char *const send,
                     bool isCRLF)
 {
     Q_D(DotAtom);
-    QString maybeDotAtom;
+    QByteArray maybeDotAtom;
     if (!parseDotAtom(scursor, send, maybeDotAtom, isCRLF)) {
         return false;
     }
@@ -1959,8 +1943,8 @@ bool ContentID::parse(const char *&scursor, const char *const send, bool isCRLF)
                 return false;
             }
 
-            // Save chars untill '>''
-            QString result;
+            // Save chars until '>''
+            QByteArray result;
             if (!parseDotAtom(scursor, send, result, false)) {
                 return false;
             }
@@ -1972,7 +1956,7 @@ bool ContentID::parse(const char *&scursor, const char *const send, bool isCRLF)
             scursor++;
             // /Almost parseAngleAddr
 
-            maybeContentId.localPart = result;
+            maybeContentId.localPart = QString::fromLatin1(result); // FIXME: just use QByteArray instead of AddrSpec in msgIdList?
             d->msgIdList.append(maybeContentId);
 
             eatCFWS(scursor, send, isCRLF);
